#import <Foundation/NSGeometry.h>
#import <Foundation/NSObject.h>

@class NSImage, NSView, NSNotificationCenter, NSArray;

@interface NSWorkspace : NSObject {
    NSNotificationCenter *notificationCenter;
    int deviceStatusCount;
    int applicationStatusCount;
}

+ (NSWorkspace *)sharedWorkspace;

- (NSNotificationCenter *)notificationCenter;


- (BOOL)openFile:(NSString *)fullPath;
- (BOOL)openFile:(NSString *)fullPath withApplication:(NSString *)appName;
- (BOOL)openFile:(NSString *)fullPath withApplication:(NSString *)appName andDeactivate:(BOOL)flag;

- (BOOL)openTempFile:(NSString *)fullPath;

- (BOOL)openFile:(NSString *)fullPath fromImage:(NSImage *)anImage at:(NSPoint)point inView:(NSView *)aView;

- (BOOL)launchApplication:(NSString *)appName;
- (BOOL)launchApplication:(NSString *)appName showIcon:(BOOL)showTile autolaunch:(BOOL)autolaunch;
- (NSString *)fullPathForApplication:(NSString *)appName;

- (BOOL)selectFile:(NSString *)fullPath inFileViewerRootedAtPath:(NSString *)rootFullpath;

- (void)findApplications;

- (void)noteFileSystemChanged;
- (BOOL)fileSystemChanged;
- (void)noteUserDefaultsChanged;
- (BOOL)userDefaultsChanged;

/* Return values for type in getInfoForFile: */

extern NSString *NSPlainFileType, *NSDirectoryFileType, *NSApplicationFileType;
extern NSString *NSFilesystemFileType, *NSShellCommandFileType;

- (BOOL)getInfoForFile:(NSString *)fullPath application:(NSString **)appName type:(NSString **)type;

- (NSImage *)iconForFile:(NSString *)fullPath;
- (NSImage *)iconForFileType:(NSString *)fileType;

- (BOOL)getInfoForFileSystemAt:(NSString *)fullPath isRemovable:(BOOL *)removableFlag isWritable:(BOOL *)writableFlag isUnmountable:(BOOL *)unmountableFlag description:(char **)description type:(char **)fileSystemType;

extern NSString *NSWorkspaceMoveOperation;
extern NSString *NSWorkspaceCopyOperation;
extern NSString *NSWorkspaceLinkOperation;
extern NSString *NSWorkspaceCompressOperation;
extern NSString *NSWorkspaceDecompressOperation;
extern NSString *NSWorkspaceEncryptOperation;
extern NSString *NSWorkspaceDecryptOperation;
extern NSString *NSWorkspaceDestroyOperation;
extern NSString *NSWorkspaceRecycleOperation;
extern NSString *NSWorkspaceDuplicateOperation;

- (BOOL)performFileOperation:(NSString *)operation source:(NSString *)source destination:(NSString *)destination files:(NSArray *)files tag:(int *)tag;

- (BOOL)unmountAndEjectDeviceAtPath:(NSString *)path;
- (int)extendPowerOffBy:(int)requested;

- (void)slideImage:(NSImage *)image from:(NSPoint)fromPoint to:(NSPoint)toPoint;

- (void)hideOtherApplications;

- (BOOL)getMountedRemovableMedia:(char **)mountedList;
- (BOOL)mountNewRemovableMedia:(char **)newlyMountedList;
- (void)checkForRemovableMedia;

@end

/* Notifications (through the special notification center) */
extern NSString *NSWorkspaceDidLaunchApplicationNotification;
extern NSString *NSWorkspaceDidMountNotification;
extern NSString *NSWorkspaceDidPerformFileOperationNotification;
extern NSString *NSWorkspaceDidTerminateApplicationNotification;
extern NSString *NSWorkspaceDidUnmountNotification;
extern NSString *NSWorkspaceWillLaunchApplicationNotification;
extern NSString *NSWorkspaceWillPowerOffNotification;
extern NSString *NSWorkspaceWillUnmountNotification;
